'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.signOut = exports.signIn = exports.fetchCredentials = undefined;

var _awsSdk = require('aws-sdk');

var _amazonCognitoIdentityJs = require('amazon-cognito-identity-js');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var _window = window,
    aws = _window.aws;


var getPool = function getPool() {
  return new _amazonCognitoIdentityJs.CognitoUserPool({
    UserPoolId: aws.cognitoUserPoolId,
    ClientId: aws.cognitoUserPoolClientId
  });
};

var fetchCredentials = exports.fetchCredentials = function fetchCredentials() {
  return function (dispatch) {
    var user = getPool(aws).getCurrentUser();

    dispatch({
      type: 'FETCH_CREDENTIALS'
    });

    if (user) {
      user.getSession(function (err, session) {
        if (err) {
          dispatch({
            type: 'FETCH_CREDENTIALS_FAILURE'
          });
        } else {
          createCredentials(user.username, session, function (credentials) {
            return dispatch({
              type: 'FETCH_CREDENTIALS_SUCCESS',
              credentials: credentials
            });
          });
        }
      });
    } else {
      dispatch({
        type: 'FETCH_CREDENTIALS_FAILURE'
      });
    }
  };
};

var signIn = exports.signIn = function signIn(username, password) {
  return function (dispatch) {
    dispatch({
      type: 'SIGN_IN'
    });

    var user = new _amazonCognitoIdentityJs.CognitoUser({
      Username: username,
      Pool: getPool()
    });

    var details = new _amazonCognitoIdentityJs.AuthenticationDetails({
      Username: username,
      Password: password
    });

    user.authenticateUser(details, {
      onSuccess: function onSuccess(session) {
        createCredentials(username, session, function (credentials) {
          return dispatch({
            type: 'SIGN_IN_SUCCESS',
            credentials: credentials,
            username: username
          });
        });
      },

      onFailure: function onFailure(error) {
        dispatch({
          type: 'SIGN_IN_ERROR',
          error: error
        });
      }
    });
  };
};

var signOut = exports.signOut = function signOut() {
  return function (dispatch) {
    var user = getPool().getCurrentUser();

    if (user) {
      user.signOut();
    }

    dispatch({
      type: 'SIGN_OUT'
    });
  };
};

function createCredentials(username, session, callback) {
  var cognitoRegion = aws.cognitoUserPoolId.split('_')[0];
  var cognitoProviderName = 'cognito-idp.' + cognitoRegion + '.amazonaws.com/' + aws.cognitoUserPoolId;

  var credentials = new _awsSdk.CognitoIdentityCredentials({
    IdentityPoolId: aws.cognitoIdentityPoolId,
    LoginId: username,
    Logins: _defineProperty({}, cognitoProviderName, session.getIdToken().getJwtToken())
  });

  credentials.cognito = new _awsSdk.CognitoIdentity({
    region: cognitoRegion,
    params: credentials.params
  });

  credentials.refresh(function (err) {
    if (err) {
      console.log(err);
    } else {
      callback(credentials);
    }
  });
}