'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.releaseVersion = exports.upload = exports.build = exports.publish = exports.toggleEditing = exports.isDevelopment = exports.isBrowser = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash');

var _awsSdk = require('aws-sdk');

var _renderRoutes = require('../renderRoutes');

var _renderRoutes2 = _interopRequireDefault(_renderRoutes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var isBrowser = exports.isBrowser = typeof document !== 'undefined';
var isDevelopment = exports.isDevelopment = process.env.NODE_ENV !== 'production';

var toggleEditing = exports.toggleEditing = function toggleEditing() {
  return {
    type: 'TOGGLE_EDITING'
  };
};

var publish = exports.publish = function publish(_ref) {
  var config = _ref.config,
      content = _ref.content,
      scripts = _ref.scripts,
      credentials = _ref.credentials,
      uploads = _ref.uploads;
  return function (dispatch) {
    dispatch({
      type: 'PUBLISH'
    });

    var version = Date.now();

    var promise = new Promise(function (resolve) {
      return resolve({
        files: []
      });
    });

    if (isBrowser && isDevelopment) {
      promise = dispatch(build());
    }

    promise.then(function (buildResult) {
      var finalScripts = buildResult.scripts || scripts;

      var contentFile = {
        path: 'content.json',
        data: new window.Blob([JSON.stringify(content)], {
          type: 'application/json'
        })
      };

      (0, _renderRoutes2.default)({
        config: config,
        content: content,
        version: version,
        scripts: finalScripts
      }, function (err, renderedRoutes) {
        if (err) {
          return console.error('Error rendering routes:', err);
        }

        var renderedFiles = (0, _lodash.map)(renderedRoutes, function (html, path) {
          return {
            path: path,
            data: new window.Blob([html], {
              type: 'text/html'
            })
          };
        });

        var files = [contentFile].concat(_toConsumableArray(uploads), _toConsumableArray(buildResult.files), _toConsumableArray(renderedFiles));

        dispatch(upload(files, credentials, version)).then(function () {
          dispatch(releaseVersion(credentials, version)).then(function () {
            dispatch({
              type: 'PUBLISH_SUCCESS'
            });
          });
        });
      });
    });
  };
};

var build = exports.build = function build() {
  return function (dispatch) {
    dispatch({
      type: 'BUILD'
    });

    return new Promise(function (resolve, reject) {
      var request = new window.XMLHttpRequest();

      request.open('post', '/build');

      request.addEventListener('readystatechange', function () {
        if (request.readyState === window.XMLHttpRequest.DONE) {
          if (request.status === 200) {
            var result = JSON.parse(request.responseText);
            var stats = result.stats;
            var clientPath = '' + stats.publicPath + stats.assetsByChunkName.client;
            var scripts = [clientPath];

            var files = result.files.map(function (file) {
              var byteString = window.atob(file.data);
              var content = new Uint8Array(byteString.length);

              for (var i = 0; i < byteString.length; i++) {
                content[i] = byteString.charCodeAt(i);
              }

              return _extends({}, file, {
                data: new window.Blob([content], {
                  type: file.type
                })
              });
            });

            dispatch({
              type: 'BUILD_SUCCESS'
            });

            resolve({
              scripts: scripts,
              files: files
            });
          }
        }
      });

      request.send();
    });
  };
};

var upload = exports.upload = function upload(files, credentials, version) {
  return function (dispatch) {
    dispatch({
      type: 'UPLOAD'
    });

    var uploads = files.map(function (file) {
      return uploadFile(file, credentials);
    });

    return Promise.all(uploads).then(function () {
      return dispatch({
        type: 'UPLOAD_SUCCESS'
      });
    });
  };
};

var releaseVersion = exports.releaseVersion = function releaseVersion(credentials, version) {
  return function (dispatch) {
    dispatch({
      type: 'RELEASE'
    });

    return new Promise(function (resolve, reject) {
      var s3 = new _awsSdk.S3({
        credentials: credentials,
        region: window.aws.s3Region
      });

      console.log('Releasing version', version);
      console.log('Updating bucket website config...');

      s3.putBucketWebsite({
        Bucket: window.aws.s3Bucket,
        WebsiteConfiguration: {
          IndexDocument: {
            Suffix: version + '-index.html'
          },
          ErrorDocument: {
            Key: version + '-404.html'
          }
        }
      }, function (err, data) {
        if (err) {
          dispatch({
            type: 'RELEASE_ERROR'
          });

          reject();
        } else {
          dispatch({
            type: 'RELEASE_SUCCESS'
          });

          resolve();
        }
      });
    });
  };
};

function uploadFile(file, credentials) {
  var key = file.path;

  var s3 = new _awsSdk.S3({
    credentials: credentials,
    region: window.aws.s3Region
  });

  var params = {
    Bucket: window.aws.s3Bucket,
    Key: key,
    Body: file.data,
    ContentType: file.type || file.data.type
  };

  console.log('Uploading', key);

  return new Promise(function (resolve, reject) {
    s3.putObject(params, function (err, res) {
      if (err) {
        reject(err);
      } else {
        console.log('Done', key);
        resolve(key);
      }
    });
  });
}