'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getImageVariations;

var _lodash = require('lodash');

var _fileToImage = require('./fileToImage');

var _fileToImage2 = _interopRequireDefault(_fileToImage);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getWidths(width) {
  var interval = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 320;

  var count = Math.max(1, Math.floor(width / interval));
  var sizes = (0, _lodash.times)(count, function (i) {
    return (i + 1) * interval;
  });

  sizes[sizes.length - 1] = width;

  return sizes;
}

function hasTransparancy(image) {
  var canvas = document.createElement('canvas');
  var context = canvas.getContext('2d');

  canvas.width = image.width;
  canvas.height = image.height;

  context.drawImage(image, 0, 0, image.width, image.height);

  var data = context.getImageData(0, 0, context.canvas.width, context.canvas.height).data;

  for (var i = 0; i < data.length; i += 4) {
    if (data[i + 3] < 255) {
      return true;
    }
  }

  return false;
}

function createVariation(image, width) {
  var type = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'image/jpeg';
  var quality = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0.8;

  return new Promise(function (resolve, reject) {
    var canvas = document.createElement('canvas');
    var context = canvas.getContext('2d');

    canvas.width = width;
    canvas.height = Math.round(width * (image.height / image.width));

    context.drawImage(image, 0, 0, canvas.width, canvas.height);

    return canvas.toBlob(function (blob) {
      return resolve({
        width: canvas.width,
        height: canvas.height,
        image: image,
        blob: blob
      });
    }, type, quality);
  });
}

function getImageVariations(file, callback) {
  (0, _fileToImage2.default)(file, function (image) {
    var widths = getWidths(image.width);

    var type = file.type;

    if (type === 'image/png' && !hasTransparancy(image)) {
      console.log('image/png image doesn’t have transparancy. Converting to image/jpeg');
      type = 'image/jpeg';
    }

    var variationPromises = widths.map(function (width) {
      return createVariation(image, width, type);
    });

    Promise.all(variationPromises).then(function (variations) {
      return callback(variations);
    });
  });
}