'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _slate = require('slate');

var _selectionPosition = require('selection-position');

var _selectionPosition2 = _interopRequireDefault(_selectionPosition);

var _ContentContainer = require('../ContentContainer');

var _ContentContainer2 = _interopRequireDefault(_ContentContainer);

var _EditIndicator = require('../EditIndicator');

var _EditIndicator2 = _interopRequireDefault(_EditIndicator);

var _FormatMenu = require('./FormatMenu');

var _FormatMenu2 = _interopRequireDefault(_FormatMenu);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var styles = {
  container: {
    position: 'relative',
    display: 'block'
  },

  text: {
    position: 'relative',
    outline: 0,
    zIndex: 4
  }
};

var DEFAULT_NODE = 'paragraph';

var schema = {
  marks: {
    bold: function bold(props) {
      return _react2.default.createElement(
        'b',
        null,
        props.children
      );
    },
    italic: function italic(props) {
      return _react2.default.createElement(
        'i',
        null,
        props.children
      );
    },
    code: function code(props) {
      return _react2.default.createElement(
        'code',
        null,
        props.children
      );
    },
    underlined: function underlined(props) {
      return _react2.default.createElement(
        'u',
        null,
        props.children
      );
    }
  },

  nodes: {
    paragraph: function paragraph(props) {
      return _react2.default.createElement(
        'p',
        props.attributes,
        props.children
      );
    },
    heading1: function heading1(props) {
      return _react2.default.createElement(
        'h1',
        props.attributes,
        props.children
      );
    },
    heading2: function heading2(props) {
      return _react2.default.createElement(
        'h2',
        props.attributes,
        props.children
      );
    },
    heading3: function heading3(props) {
      return _react2.default.createElement(
        'h3',
        props.attributes,
        props.children
      );
    },
    heading4: function heading4(props) {
      return _react2.default.createElement(
        'h4',
        props.attributes,
        props.children
      );
    },
    unorderedList: function unorderedList(props) {
      return _react2.default.createElement(
        'ul',
        props.attributes,
        props.children
      );
    },
    orderedList: function orderedList(props) {
      return _react2.default.createElement(
        'ol',
        props.attributes,
        props.children
      );
    },
    listItem: function listItem(props) {
      return _react2.default.createElement(
        'li',
        props.attributes,
        props.children
      );
    },
    link: function link(props) {
      var data = props.node.data;

      var href = data.get('href');
      return _react2.default.createElement(
        'a',
        _extends({ href: href }, props.attributes),
        props.children
      );
    }
  }
};

var EditText = function (_React$Component) {
  _inherits(EditText, _React$Component);

  function EditText() {
    var _ref;

    var _temp, _this, _ret;

    _classCallCheck(this, EditText);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_ref = EditText.__proto__ || Object.getPrototypeOf(EditText)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      slateState: _this.getSlateState()
    }, _this.componentDidMount = function () {
      _this.updateFormatMenu();
    }, _this.componentDidUpdate = function () {
      _this.updateFormatMenu();
    }, _this.onFormatMenuOpen = function (portal) {
      _this.setState({ menuWrapper: portal.firstChild });
    }, _this.updateFormatMenu = function () {
      var _this$state = _this.state,
          menuWrapper = _this$state.menuWrapper,
          slateState = _this$state.slateState;


      if (!menuWrapper) return;

      if (slateState.isBlurred || slateState.isCollapsed) {
        menuWrapper.style.display = 'none';
        return;
      }

      function getMenuPosition() {
        var rect = (0, _selectionPosition2.default)();

        if (window.innerWidth - rect.left < menuWrapper.offsetWidth / 2) {
          // Outside right screen
          return {
            top: rect.top + window.scrollY - menuWrapper.offsetHeight + 'px',
            left: window.innerWidth - menuWrapper.offsetWidth + 'px'
          };
        }

        if (rect.left < menuWrapper.offsetWidth / 2 - rect.width / 2) {
          // Outside left screen
          return {
            top: rect.top + window.scrollY - menuWrapper.offsetHeight + 'px',
            left: window.scrollX + 'px'
          };
        }

        return {
          top: rect.top + window.scrollY - menuWrapper.offsetHeight + 'px',
          left: rect.left + window.scrollX - menuWrapper.offsetWidth / 2 + rect.width / 2 + 'px'
        };
      }

      menuWrapper.style.display = 'block';
      menuWrapper.style.top = getMenuPosition().top;
      menuWrapper.style.left = getMenuPosition().left;
    }, _this.hasMark = function (type) {
      var slateState = _this.state.slateState;

      return slateState.marks.some(function (mark) {
        return mark.type === type;
      });
    }, _this.hasLink = function () {
      var slateState = _this.state.slateState;

      return slateState.inlines.some(function (inline) {
        return inline.type === 'link';
      });
    }, _this.hasBlock = function (type) {
      var slateState = _this.state.slateState;

      return slateState.blocks.some(function (node) {
        return node.type === type;
      });
    }, _this.onMarkButtonClick = function (e, type) {
      e.preventDefault();
      var slateState = _this.state.slateState;


      slateState = slateState.transform().toggleMark(type).apply();

      _this.props.setValue(_slate.Raw.serialize(slateState, { terse: true }));
      _this.setState({ slateState: slateState });
    }, _this.onLinkButtonClick = function (e, type) {
      e.preventDefault();
      var slateState = _this.state.slateState;

      var hasLink = _this.hasLink();

      if (hasLink) {
        slateState = slateState.transform().unwrapInline('link').apply();
      } else if (slateState.isExpanded) {
        var href = window.prompt('Enter link URL');

        slateState = slateState.transform().wrapInline({
          type: 'link',
          data: { href: href }
        }).collapseToEnd().apply();
      } else {
        var _href = window.prompt('Enter link URL');
        var text = window.prompt('Enter link text');

        slateState = slateState.transform().insertText(text).extendBackward(text.length).wrapInline({
          type: 'link',
          data: { href: _href }
        }).collapseToEnd().apply();
      }

      _this.props.setValue(_slate.Raw.serialize(slateState, { terse: true }));
      _this.setState({ slateState: slateState });
    }, _this.onBlockButtonClick = function (e, type) {
      e.preventDefault();
      var slateState = _this.state.slateState;

      var transform = slateState.transform();
      var _slateState = slateState,
          document = _slateState.document;


      if (type !== 'unorderedList' && type !== 'orderedList') {
        var isActive = _this.hasBlock(type);
        var isList = _this.hasBlock('listItem');

        if (isList) {
          transform.setBlock(isActive ? DEFAULT_NODE : type).unwrapBlock('orderedList').unwrapBlock('unorderedList');
        } else {
          transform.setBlock(isActive ? DEFAULT_NODE : type);
        }
      } else {
        var _isList = _this.hasBlock('listItem');
        var isType = slateState.blocks.some(function (block) {
          return !!document.getClosest(block.key, function (parent) {
            return parent.type === type;
          });
        });

        if (_isList && isType) {
          transform.setBlock(DEFAULT_NODE).unwrapBlock('orderedList').unwrapBlock('unorderedList');
        } else if (_isList) {
          transform.unwrapBlock(type === 'unorderedList' ? 'orderedList' : 'unorderedList').wrapBlock(type);
        } else {
          transform.setBlock('listItem').wrapBlock(type);
        }
      }

      slateState = transform.apply();

      _this.props.setValue(_slate.Raw.serialize(slateState, { terse: true }));
      _this.setState({ slateState: slateState });
    }, _this.onChange = function (slateState) {
      _this.props.setValue(_slate.Raw.serialize(slateState, { terse: true }));
      _this.setState({ slateState: slateState });
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(EditText, [{
    key: 'getSlateState',
    value: function getSlateState() {
      if (this.props.value) {
        return _slate.Raw.deserialize(this.props.value, { terse: true });
      }
      return _slate.Raw.deserialize({
        nodes: [{
          kind: 'block',
          type: 'paragraph',
          nodes: [{
            kind: 'text',
            text: this.props.placeholder
          }]
        }]
      }, { terse: true });
    }
  }, {
    key: 'render',
    value: function render() {
      var slateState = this.state.slateState;


      return _react2.default.createElement(
        _EditIndicator2.default,
        null,
        _react2.default.createElement(
          'div',
          { style: styles.container },
          _react2.default.createElement(_FormatMenu2.default, {
            onMarkButtonClick: this.onMarkButtonClick,
            onBlockButtonClick: this.onBlockButtonClick,
            onLinkButtonClick: this.onLinkButtonClick,
            onFormatMenuOpen: this.onFormatMenuOpen,
            hasMark: this.hasMark,
            hasLink: this.hasLink,
            hasBlock: this.hasBlock
          }),
          _react2.default.createElement(_slate.Editor, {
            schema: schema,
            state: slateState,
            onChange: this.onChange.bind(this)
          })
        )
      );
    }
  }]);

  return EditText;
}(_react2.default.Component);

EditText.propTypes = {
  value: _react2.default.PropTypes.object,
  setValue: _react2.default.PropTypes.func.isRequired,
  placeholder: _react2.default.PropTypes.string
};
EditText.defaultProps = {
  placeholder: ''
};
exports.default = (0, _ContentContainer2.default)(EditText);