'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _slate = require('slate');

var _ContentContainer = require('../ContentContainer');

var _ContentContainer2 = _interopRequireDefault(_ContentContainer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var rules = [{
  serialize: function serialize(object, children) {
    if (object.kind !== 'block') return;
    switch (object.type) {
      case 'paragraph':
        return _react2.default.createElement(
          'p',
          null,
          children
        );
      case 'heading1':
        return _react2.default.createElement(
          'h1',
          null,
          children
        );
      case 'heading2':
        return _react2.default.createElement(
          'h2',
          null,
          children
        );
      case 'heading3':
        return _react2.default.createElement(
          'h3',
          null,
          children
        );
      case 'heading4':
        return _react2.default.createElement(
          'h4',
          null,
          children
        );
      case 'unorderedList':
        return _react2.default.createElement(
          'ul',
          null,
          children
        );
      case 'orderedList':
        return _react2.default.createElement(
          'ol',
          null,
          children
        );
      case 'listItem':
        return _react2.default.createElement(
          'li',
          null,
          children
        );
      case 'code':
        return _react2.default.createElement(
          'pre',
          null,
          _react2.default.createElement(
            'code',
            null,
            children
          )
        );
    }
  }
}, {
  serialize: function serialize(object, children) {
    if (object.kind !== 'inline') return;

    switch (object.type) {
      case 'link':
        return _react2.default.createElement(
          'a',
          { href: object.data.get('href') },
          children
        );
    }
  }
}, {
  serialize: function serialize(object, children) {
    if (object.kind !== 'mark') return;
    switch (object.type) {
      case 'bold':
        return _react2.default.createElement(
          'b',
          null,
          children
        );
      case 'italic':
        return _react2.default.createElement(
          'i',
          null,
          children
        );
      case 'code':
        return _react2.default.createElement(
          'code',
          null,
          children
        );
    }
  }
}];

function getHtml(state) {
  var html = new _slate.Html({ rules: rules });
  return { __html: html.serialize(state) };
}

exports.default = (0, _ContentContainer2.default)(function (_ref) {
  var value = _ref.value;

  if (!value) return null;

  var slateState = _slate.Raw.deserialize(value, { terse: true });

  return _react2.default.createElement('div', { dangerouslySetInnerHTML: getHtml(slateState) });
});