'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var React = require('react');
var isValidElement = require('react').isValidElement;
var NotFound = require('../components/NotFound').default;

// Valid configs:
// 1. Valid React element
// <Component />

// 2. Array of routes
// [{ path: '', component: '' }]

// 3. Function
// content => [{ path: '', component: '' }]

// 4. Object
// { routes: content => [{ path: '', component: '' }] }

var defaultConfig = {
  routes: function routes() {
    return [];
  },
  notFoundRoute: {
    title: '404 not found',
    component: React.createElement(NotFound, null)
  },
  template: function template(html, title) {
    return '\n      <!doctype html>\n      <html>\n      <head>\n        <meta charset="utf-8" />\n        <title>' + title + '</title>\n        <meta name="viewport" content="width=device-width, initial-scale=1" />\n      </head>\n      <body>\n        <div id="root">\n          ' + html + '\n        </div>\n      </body>\n      </html>\n    ';
  }
};

module.exports = function defaults(entry) {
  if (entry.default) {
    return defaults(entry.default);
  }

  // Valid React element
  if (isValidElement(entry)) {
    return Object.assign({}, defaultConfig, {
      routes: function routes() {
        return [{ path: '/', component: entry }];
      }
    });
  }

  // Array
  if (entry.constructor === Array) {
    return Object.assign({}, defaultConfig, {
      routes: function routes() {
        return entry;
      }
    });
  }

  // Object
  if ((typeof entry === 'undefined' ? 'undefined' : _typeof(entry)) === 'object' && entry.constructor === Object) {
    return Object.assign({}, defaultConfig, entry);
  }

  // Function
  if (typeof entry === 'function') {
    return Object.assign({}, defaultConfig, {
      routes: entry
    });
  }
};