'use strict';

var path = require('path');
// const fs = require('fs')
var MemoryFS = require('memory-fs');
var requireFromString = require('require-from-string');
var webpack = require('webpack');
var configPath = require('./path');
var configDefaults = require('./defaults');
var getWebpackConfig = require('../getWebpackConfig');

module.exports = function (callback) {
  console.log('Compiling project main...');

  var webpackConfig = Object.assign({}, getWebpackConfig('common'));

  webpackConfig.target = 'node';
  webpackConfig.output.libraryTarget = 'commonjs';

  webpackConfig.entry = {
    config: configPath
  };

  if (!webpackConfig.resolve) {
    webpackConfig.resolve = {};
  }

  webpackConfig.resolve.aliasFields = ['server'];
  webpackConfig.resolve.mainFields = ['module', 'main'];

  var compiler = webpack(webpackConfig);

  var fs = new MemoryFS();

  compiler.outputFileSystem = fs;

  compiler.run(function (err, stats) {
    if (err) {
      callback(err);
    }

    if (stats.hasErrors()) {
      callback(new Error(stats.toString()));
    }

    console.log('Done compiling project main...');

    var filename = stats.toJson().assetsByChunkName.config;
    var configPath = webpackConfig.output.path + '/' + filename;

    if (fs.existsSync(configPath)) {
      fs.readFile(configPath, 'utf-8', function (err, data) {
        if (err) {
          callback(err);
        }

        try {
          var getConfig = requireFromString(data).default;
          var config = configDefaults(getConfig);
          callback(null, config);
        } catch (e) {
          callback(e);
        }
      });
    } else {
      callback(new Error('Could not read ' + path + ' from MemoryFS filesystem'));
    }
  });
};