'use strict';

var path = require('path');
var fs = require('fs');
var isPlainObject = require('lodash').isPlainObject;
var merge = require('./merge').default;

module.exports = function getWebpackConfig(environment) {
  var baseConfig = require('./webpack.' + environment + '.config.js');
  var projectConfigPath = path.resolve(process.cwd(), 'webpack.config.js');

  var transform = function transform(config) {
    return config;
  };

  if (fs.existsSync(projectConfigPath)) {
    var projectConfig = require(projectConfigPath);

    projectConfig = projectConfig.default || projectConfig;

    if (isPlainObject(projectConfig)) {
      transform = function transform(config) {
        return merge(config, projectConfig);
      };
    } else if (typeof projectConfig === 'function') {
      transform = projectConfig;
    }
  }

  return transform(baseConfig);
};