'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = renderRoutes;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Provider = require('./components/Provider');

var _Provider2 = _interopRequireDefault(_Provider);

var _server = require('react-dom/server');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function renderWithProvider(component, config, content, scripts) {
  return (0, _server.renderToStaticMarkup)(_react2.default.createElement(
    _Provider2.default,
    { content: content, config: config, assets: {}, scripts: scripts },
    component
  ));
}

function sanitizeJSON(string) {
  return string.replace(/\u2028/g, '\\u2028').replace(/\u2029/g, '\\u2029').replace(/'/g, '\\u0027');
}

function renderRoutes(_ref, callback) {
  var config = _ref.config,
      content = _ref.content,
      scripts = _ref.scripts,
      version = _ref.version;

  var prefix = version ? version + '-' : '';
  var routes = config.routes(content);
  var template = config.template;
  var files = {};

  try {
    // Route files
    routes.forEach(function (route) {
      if (route.path.slice(-1) !== '/') {
        callback(new Error('Route path \'' + route.path + '\' is missing trailing slash.'));
      }

      var filepath = ('/' + route.path + '/' + prefix + 'index.html').split('/').filter(function (v) {
        return !!v;
      }).join('/');
      var html = renderWithProvider(route.component, config, content, scripts);

      var injectBefore = '</body>';
      var injection = '\n        <script>\n          window.content = \'' + sanitizeJSON(JSON.stringify(content)) + '\';\n          window.scripts = \'' + JSON.stringify(scripts) + '\';\n        </script>\n        ' + scripts.map(function (script) {
        return '\n          <script src="' + script + '"></script>\n        ';
      }) + '\n      ';

      var rendered = template(html, route.title);

      if (rendered.indexOf(injectBefore) === -1) {
        rendered += injection;
      } else {
        rendered = rendered.replace(injectBefore, '' + injection + injectBefore);
      }

      files[filepath] = rendered;
    });

    // 404 file
    if (config.notFoundRoute) {
      var html = renderWithProvider(config.notFoundRoute.component, config, content, scripts);
      files[prefix + '404.html'] = template(html, config.notFoundRoute.title, content);
    }

    callback(null, files);
  } catch (err) {
    callback(err);
  }
}